/*
* Copyright (c) 2006-2011 Massachusetts General Hospital 
 * All rights reserved. This program and the accompanying materials 
* are made available under the terms of the i2b2 Software License v2.1 
 * which accompanies this distribution. 
 * 
 * Contributors:
 *     Lori Phillips - initial API and implementation
 */
package edu.harvard.i2b2.eclipse.plugins.vista.table.views;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.params.HttpMethodParams;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;



public class SnpConservationImageProvider {
	private static SnpConservationImageProvider content;
	
	
	private SnpConservationImageProvider() {
	
	}
	public static synchronized SnpConservationImageProvider getInstance() {
		if (content != null) {
			return content;
		}
		content = new SnpConservationImageProvider();
		return content;
	}

	
	private Image getImage(SnpModel snp){
		// first get snp positions
		getPosition(snp);
		
		// Now get image map for this snp
		String url = "http://pipeline.lbl.gov";
		String method =	"/servlet/gpvista2/?output=plot&regular=0&guidelines=0&left_labels=0&contigs=0&width=200&height=10&run=80&org=2&pos=";

		method = method + snp.getGenomicData().getChromosomeLocation().getChromosome() + ":"
		+ (snp.getGenomicData().getChromosomeLocation().getStart()-100) + "-" 
		+ (snp.getGenomicData().getChromosomeLocation().getEnd() + 100)
		+ "&base=" + "hg18";
		
	
		Image image = null;

		HttpClient client = new HttpClient();
		GetMethod getMethod = new GetMethod(url+method);

		// Provide custom retry handler is necessary
		getMethod.getParams().setParameter(HttpMethodParams.RETRY_HANDLER, 
				new DefaultHttpMethodRetryHandler(3, false));

		try {
			// Execute the method.
			int statusCode = client.executeMethod(getMethod);
			if (statusCode != HttpStatus.SC_OK) {
				displayErrorMessage("gpvista url failed: " + getMethod.getStatusLine());
				getMethod.releaseConnection();
				return image;
			}

			// Read the response body.

			InputStream in = getMethod.getResponseBodyAsStream();
			image = new Image(Display.getCurrent(), in);
			

		} catch (HttpException e) {
			displayErrorMessage("Fatal protocol violation: " + e.getMessage());
		} catch (IOException e) {
			displayErrorMessage("Fatal transport error: " + e.getMessage());
		} finally {
			// Release the connection.
			getMethod.releaseConnection();
		}  

	
		return image;
	}	
	
	public Thread getConservationImage(SnpModel snp, TableViewer viewer, Display display){
		final SnpConservationImageProvider provider = this;
		final SnpModel theModel = snp;
		final TableViewer theViewer = viewer;
		final Display theDisplay = display;
		return new Thread() {
			@Override
			public void run(){
				try {
					theModel.setConservationImage(provider.getImage(theModel));
				} catch (Exception e) {
					// TODO Auto-generated catch block
	//				System.setProperty("statusMessage", e.getMessage());					
				}
				theDisplay.syncExec(new Runnable() {
					public void run() {
						theViewer.update(theModel, null);
					}
				});
			}
		};
	}
	
	

	public void getMap(){
		int snpStartLow = 0;
		int snpEndHigh = 0;
		String chromosome = null;
		String base = "hg18";
		String snpUrlList = "";

		List <SnpModel> snpsOfInterest = SnpModelProvider.getInstance().getEnabledSnps();

		// for each snp of Interest get location 

		Iterator<SnpModel> snpsIterator = snpsOfInterest.iterator();		
		while(snpsIterator.hasNext()){
			SnpModel snp = (SnpModel) snpsIterator.next();

			if(snp.getGenomicData().getChromosomeLocation().getStart() != 0){
				if ((snp.getGenomicData().getChromosomeLocation().getStart() < snpStartLow) || (snpStartLow == 0)) {
					snpStartLow = snp.getGenomicData().getChromosomeLocation().getStart();
				}
				if (snp.getGenomicData().getChromosomeLocation().getEnd() > snpEndHigh){
					snpEndHigh = snp.getGenomicData().getChromosomeLocation().getEnd();
				}
				if(chromosome == null){
					chromosome = snp.getGenomicData().getChromosomeLocation().getChromosome();
				}
			}
			else if(snp.getGenomicData().getChromosomeLocation().getStart() == 0){

						base = snp.getGenomicData().getSequenceVariantLocation().getReferenceGenomeVersion();

				if((snp.getGenomicData().getSequenceVariantLocation().getGeneName() != null)
						&& (snp.getGenomicData().getSequenceVariantLocation().getFlankingSeq5() != null)
						&& (snp.getGenomicData().getSequenceVariantLocation().getFlankingSeq3() != null))
				{
					getPosition(snp);

					chromosome = snp.getGenomicData().getChromosomeLocation().getChromosome();


					if(snpStartLow == 0){
						snpStartLow = snp.getGenomicData().getChromosomeLocation().getStart();
					}
					else if (snp.getGenomicData().getChromosomeLocation().getStart() < snpStartLow){
						snpStartLow = snp.getGenomicData().getChromosomeLocation().getStart();
					}

					if (snp.getGenomicData().getChromosomeLocation().getEnd() > snpEndHigh){
						snpEndHigh = snp.getGenomicData().getChromosomeLocation().getEnd();
					}
				}	

			}
			try {
				snpUrlList = snpUrlList + "&snp=" +  URLEncoder.encode(snp.getGenomicData().getSequenceVariant().getHGVSName()+","+snp.getGenomicData().getChromosomeLocation().getStart(),"UTF-8");
			} catch (UnsupportedEncodingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	

	// Now get map for entire region and display in browser.
		String url1 = "http://pipeline.lbl.gov";
		String method1 =	"/servlet/gpvista2/?output=plot&regular=0&guidelines=0&left_labels=1&contigs=0&width=750&height=40&run=80&order=2,3,11,23,362,696&pos=";

		String url2="http://hazelton.lbl.gov/flankmap/map/snp/image?width=750&position=";  //was 635
		
		if((chromosome != null) && (snpStartLow != 0) & (snpEndHigh != 0)){
			method1 = method1 + chromosome + ":" + (snpStartLow-200) + "-" + (snpEndHigh+200) + "&base=" + base ; 
			//		log.debug(method);

			String method2 = null;
			try {
				method2 =  URLEncoder.encode(chromosome + ":" + (snpStartLow-200) + "-" + (snpEndHigh+200),"UTF-8") + snpUrlList+ "&shownames=true";
			} catch (UnsupportedEncodingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			final String mapUrl = url1 + method1;
			final String tickUrl = url2 + method2;
			Display display=Display.getCurrent();
			final Shell myShell=display.getActiveShell();
			display.asyncExec(new Runnable() {

				public void run() {

					new ImageViewer().run(mapUrl, tickUrl, myShell);
									
				//	new HelpBrowser().run(mapUrl,myShell);
				}

			}); 
		}
	}




	private void displayErrorMessage(String message){
		MessageBox mBox = new MessageBox(Display.getCurrent().getActiveShell(),SWT.OK);
		mBox.setText("Vista Message");
		mBox.setMessage(message + "\n Please contact your administrator\n if this is a problem");
		mBox.open();
	}

	private void getPosition(SnpModel snp){

		// Get position
		String url = "http://hazelton.lbl.gov" ;
		String map = "/flankmap/map?"; 
		String	set =	"dataset=hg18";
		String gene = "&gene=" + snp.getGenomicData().getSequenceVariantLocation().getGeneName();
		String flank_5 ="&flank_seq_5=" + snp.getGenomicData().getSequenceVariantLocation().getFlankingSeq5();
		String flank_3 = "&flank_seq_3="  + snp.getGenomicData().getSequenceVariantLocation().getFlankingSeq3();

		
		String flankmap = map + set + gene + flank_5 + flank_3;

		String result = getLocation(url , flankmap);
		SAXBuilder parser = new SAXBuilder();
		java.io.StringReader xmlStringReader = new java.io.StringReader(result);

		try {
			org.jdom.Document flankMapDoc = parser.build(xmlStringReader);
			org.jdom.Element flankMapXml = flankMapDoc.getRootElement();
			Element posXml = (Element) flankMapXml.getChild("ref_position");

			String pos = posXml.getText().replace(",", "");
			String[] pieces = pos.split(":");	
			//	chr11:47329420-47329440";

			snp.getGenomicData().getChromosomeLocation().setChromosome(pieces[0]);
			String[] endpoints = pieces[1].split("-");
			snp.getGenomicData().getChromosomeLocation().setStart(Integer.parseInt(endpoints[0]));
			snp.getGenomicData().getChromosomeLocation().setEnd(Integer.parseInt(endpoints[1]));
		}	catch(Exception e1) {
			//
		}
	}
	
	private String getLocation(String url, String meth){

		String flankMap = "";

		HttpClient client = new HttpClient();
		GetMethod method = new GetMethod(url+meth);

		// Provide custom retry handler is necessary
		method.getParams().setParameter(HttpMethodParams.RETRY_HANDLER, 
				new DefaultHttpMethodRetryHandler(3, false));

		try {
			// Execute the method.
			int statusCode = client.executeMethod(method);
			if (statusCode != HttpStatus.SC_OK) {
				displayErrorMessage("flankmap error : " +method.getStatusLine().getReasonPhrase());
				method.releaseConnection();
				return flankMap;
			}

			// Read the response body.

			byte[] responseBody = method.getResponseBody();

			// Deal with the response.
			// Use caution: ensure correct character encoding and is not binary data
			flankMap = new String(responseBody);


		} catch (HttpException e) {
			displayErrorMessage("Fatal protocol violation: " + e.getMessage());
		} catch (IOException e) {
			displayErrorMessage("Fatal transport error: " + e.getMessage());
		} finally {
			// Release the connection.
			method.releaseConnection();
		}  

		return flankMap;
	}
}