/*
* Copyright (c) 2006-2011 Massachusetts General Hospital 
 * All rights reserved. This program and the accompanying materials 
* are made available under the terms of the i2b2 Software License v2.1 
 * which accompanies this distribution. 
 * 
 * Contributors:
 *     Lori Phillips - initial API and implementation
 */
package edu.harvard.i2b2.eclipse.plugins.vista.table.views;

import java.util.List;

import javax.xml.bind.JAXBElement;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.TextTransfer;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.input.DOMBuilder;
import org.jdom.output.DOMOutputter;
import org.eclipse.swt.widgets.Display;
import edu.harvard.i2b2.common.util.jaxb.JAXBUnWrapHelper;
import edu.harvard.i2b2.common.util.jaxb.JAXBUtil;
import edu.harvard.i2b2.common.util.jaxb.JAXBUtilException;
import edu.harvard.i2b2.crcxmljaxb.datavo.dnd.DndType;
import edu.harvard.i2b2.crcxmljaxb.datavo.vdo.ConceptsType;
import edu.harvard.i2b2.crcxmljaxb.datavo.vdo.GenomicMetadataType;
import edu.harvard.i2b2.crcxmljaxb.datavo.vdo.GetTermInfoType;
import edu.harvard.i2b2.crcxmljaxb.datavo.vdo.XmlValueType;
import edu.harvard.i2b2.eclipse.plugins.vista.util.VistaJAXBUtil;
import edu.harvard.i2b2.eclipse.plugins.vista.ws.GetTermInfoResponseMessage;
import edu.harvard.i2b2.eclipse.plugins.vista.ws.OntServiceDriver;

public class SnpDropListener  implements DropTargetListener{

	private Log log = LogFactory.getLog(SnpDropListener.class.getName());	
	private TableViewer viewer;
	
	public SnpDropListener(TableViewer theViewer){
		viewer = theViewer;
	}
	
	public void dragEnter(DropTargetEvent event) {
		TextTransfer textTransfer = TextTransfer.getInstance();
		if(textTransfer.isSupportedType(event.currentDataType)){
			if(event.detail == DND.DROP_DEFAULT)
				event.detail = DND.DROP_COPY;
		}
	}

	public void dragLeave(DropTargetEvent event) {
		// TODO Auto-generated method stub
		
	}

	public void dragOperationChanged(DropTargetEvent event) {
		// TODO Auto-generated method stub
		
	}

	public void dragOver(DropTargetEvent event) {
		// TODO Auto-generated method stub
		
	}

	public void drop(DropTargetEvent event) {
		if (event.data == null) {
    		event.detail = DND.DROP_NONE;
    		return;
    	}
    	 getDndContent((String)event.data);
	}

	public void dropAccept(DropTargetEvent event) {
		// TODO Auto-generated method stub
		
	}
	
	private void getDndContent(String xmlstr){
		try {
			JAXBUtil jaxbUtil = VistaJAXBUtil.getJAXBUtil();
			JAXBElement jaxbElement = jaxbUtil.unMashallFromString(xmlstr);
			DndType dndType = (DndType)jaxbElement.getValue();

			if(dndType != null){
				//		log.info("dndType is null");

				ConceptsType conceptsType = (ConceptsType) new JAXBUnWrapHelper().getObjectByClass(dndType.getAny(),
						ConceptsType.class);

				if(conceptsType != null){
					getMetadata(conceptsType, viewer).start();
				}
			}		  


		}catch (JAXBUtilException e) {
			log.error("Problem with dropped concept");

		}
	}


	
	private Thread getMetadata(ConceptsType conceptsType, TableViewer viewer)
	{	
		final Display theDisplay = Display.getCurrent();
		final TableViewer theViewer = viewer;
		final List<SnpModel> list = SnpModelProvider.getInstance().getSnps();
		final GetTermInfoType term = new GetTermInfoType();
		term.setSelf(conceptsType.getConcept().get(0).getKey());
		term.setBlob(true);
		term.setSynonyms(false);
		term.setType("core");
		return new Thread() {
			public void run() {
				String responseXml = null;

				try {
					responseXml = OntServiceDriver.getTermInfo(term);

					XmlValueType xml =  null;
					GenomicMetadataType genomicType = null;
					GetTermInfoResponseMessage msg = new GetTermInfoResponseMessage();
					msg.processResult(responseXml);
					ConceptsType allConcepts = msg.doReadConcepts();   	  		
					xml = (XmlValueType) allConcepts.getConcept().get(0).getMetadataxml();

					org.w3c.dom.Element ele = xml.getAny().get(0);
					DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
					DocumentBuilder builder = factory.newDocumentBuilder();
					org.w3c.dom.Document doc1 = builder.newDocument();
					doc1.appendChild( doc1.importNode(ele,true)); 
					org.jdom.input.DOMBuilder builder2 = new DOMBuilder();
					org.jdom.Document jdomDoc = builder2.build(doc1);


					Namespace namespace
					= Namespace.getNamespace("http://www.i2b2.org/xsd/cell/ont/1.1/");

					jdomDoc.getRootElement().setNamespace(namespace);

					DOMOutputter outputter = new DOMOutputter();
					JAXBUtil jaxbUtil = VistaJAXBUtil.getJAXBUtil();
					JAXBElement jaxbElement = jaxbUtil.unMashallFromDocument(outputter.output(jdomDoc));
					genomicType = (GenomicMetadataType)jaxbElement.getValue();
					SnpModel snp = new SnpModel();
					snp.setGenomicData(genomicType);
					snp.setShowPosition(false);
					list.add(snp);
				} catch (JDOMException e) {
					log.error("Problem with concept's GenomicMetadata");
				} catch (ParserConfigurationException e) {
					log.error("Problem with concept's GenomicMetadata");
					
				} catch (Exception e) {
					log.error("Problem with ONT getTermInfo call to get GenomicMetadata");
					log.error(e.getMessage());
				}
				theDisplay.syncExec(new Runnable(){
					public void run(){
						theViewer.setInput(list);
						theViewer.refresh();
					}
				});
			}
		};
	}


}
