/*
* Copyright (c) 2006-2011 Massachusetts General Hospital 
 * All rights reserved. This program and the accompanying materials 
* are made available under the terms of the i2b2 Software License v2.1 
 * which accompanies this distribution. 
 * 
 * Contributors:
 *     Lori Phillips - initial API and implementation
 */
package edu.harvard.i2b2.eclipse.plugins.vista.table.views;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.params.HttpMethodParams;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.CheckboxCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;


public class VistaCellEditingSupport extends EditingSupport {
		private CellEditor editor;
		private int column;

		public VistaCellEditingSupport(ColumnViewer viewer, int column) {
			super(viewer);

			// Create the correct editor based on the column index
			switch (column) {
			case 4:
			editor = new CheckboxCellEditor(null, SWT.CHECK | SWT.READ_ONLY);
				break;
			case 0:
				editor = new CheckboxCellEditor(null, SWT.CHECK | SWT.READ_ONLY);
				break;
			default:
				editor = null;
				//				editor = new TextCellEditor(((TableViewer) viewer).getTable());
			}
			this.column = column;
		}

		@Override
		protected boolean canEdit(Object element) {
			return true;
		}

		@Override
		protected CellEditor getCellEditor(Object element) {
			return editor;
		}

		@Override
		protected Object getValue(Object element) {
			SnpModel snp = (SnpModel) element;

			switch (this.column) {
	/*		case 0:
				return person.getFirstName();
			case 1:
				return person.getLastName();
	*/		case 4:
				SnpConservationImageProvider.getInstance().getMap();
				return true;
	
			case 0:
				return snp.isShowPosition();
			default:
				break;
			}
			return null;
		}

		@Override
		protected void setValue(Object element, Object value) {
			SnpModel snp = (SnpModel) element;

			switch (this.column) {
	/*		case 0:
				pers.setFirstName(String.valueOf(value));
				break;
			case 1:
				pers.setLastName(String.valueOf(value));
				break;
			case 2:
				if (((Integer) value) == 0) {
					pers.setGender("male");
				} else {
					pers.setGender("female");
				}
				break;
	*/
			case 0:
				snp.setShowPosition((Boolean) value);
				break;
			default:
				break;
			}

			getViewer().update(element, null);
		}


		private void getMap(){
			int snpStartLow = 0;
			int snpEndHigh = 0;
			String chromosome = null;
			String base = "hg18";

			List <SnpModel> snpsOfInterest = SnpModelProvider.getInstance().getEnabledSnps();

			// for each snp of Interest get location 

			Iterator<SnpModel> snpsIterator = snpsOfInterest.iterator();		
			while(snpsIterator.hasNext()){
				SnpModel snp = (SnpModel) snpsIterator.next();
				if(snp.getGenomicData().getChromosomeLocation().getStart() != 0){
					if ((snp.getGenomicData().getChromosomeLocation().getStart() < snpStartLow) || (snpStartLow == 0)) {
						snpStartLow = snp.getGenomicData().getChromosomeLocation().getStart();
					}
					if (snp.getGenomicData().getChromosomeLocation().getEnd() > snpEndHigh){
						snpEndHigh = snp.getGenomicData().getChromosomeLocation().getEnd();
					}
					if(chromosome == null){
						chromosome = snp.getGenomicData().getChromosomeLocation().getChromosome();
					}
				}
				else if(snp.getGenomicData().getChromosomeLocation().getStart() == 0){

					//		base = snp.getGenomicData().getSequenceVariantLocation().getReferenceGenomeVersion();

					if((snp.getGenomicData().getSequenceVariantLocation().getGeneName() != null)
							&& (snp.getGenomicData().getSequenceVariantLocation().getFlankingSeq5() != null)
							&& (snp.getGenomicData().getSequenceVariantLocation().getFlankingSeq3() != null))
					{
						// Get position
						String url = "http://hazelton.lbl.gov" ;
						String map = "/flankmap/map?"; 
						String	set =	"dataset=hg18";
						String gene = "&gene=" + snp.getGenomicData().getSequenceVariantLocation().getGeneName();
						String flank_5 ="&flank_seq_5=" + snp.getGenomicData().getSequenceVariantLocation().getFlankingSeq5();
						String flank_3 = "&flank_seq_3="  + snp.getGenomicData().getSequenceVariantLocation().getFlankingSeq3();

						String flankmap = map + set + gene + flank_5 + flank_3;

						String result = getLocation(url , flankmap);
						SAXBuilder parser = new SAXBuilder();
						java.io.StringReader xmlStringReader = new java.io.StringReader(result);

						try {
							org.jdom.Document flankMapDoc = parser.build(xmlStringReader);
							org.jdom.Element flankMapXml = flankMapDoc.getRootElement();
							Element posXml = (Element) flankMapXml.getChild("ref_position");

							String pos = posXml.getText().replace(",", "");
							String[] pieces = pos.split(":");	
							//	chr11:47329420-47329440";

							chromosome = pieces[0];
							snp.getGenomicData().getChromosomeLocation().setChromosome(pieces[0]);
							String[] endpoints = pieces[1].split("-");
							snp.getGenomicData().getChromosomeLocation().setStart(Integer.parseInt(endpoints[0]));
							snp.getGenomicData().getChromosomeLocation().setEnd(Integer.parseInt(endpoints[1]));

							if(snpStartLow == 0){
								snpStartLow = snp.getGenomicData().getChromosomeLocation().getStart();
							}
							else if (snp.getGenomicData().getChromosomeLocation().getStart() < snpStartLow){
								snpStartLow = snp.getGenomicData().getChromosomeLocation().getStart();
							}

							if (snp.getGenomicData().getChromosomeLocation().getEnd() > snpEndHigh){
								snpEndHigh = snp.getGenomicData().getChromosomeLocation().getEnd();
							}
						}	
						catch(Exception e1) {
							//
						}
					}
				}
			}

			// Now get map for entire region and display in browser.
			String url = "http://pipeline.lbl.gov";
			String method =	"/servlet/gpvista2/?output=plot&regular=0&guidelines=0&left_labels=1&contigs=0&width=800&height=40&run=80&order=2,3,11,23,362,696&pos=";

			if((chromosome != null) && (snpStartLow != 0) & (snpEndHigh != 0)){
				method = method + chromosome + ":" + (snpStartLow-100) + "-" + (snpEndHigh+100) + "&base=" + base ; 
				//		log.debug(method);

				final String mapUrl = url + method;

				Display display=Display.getCurrent();
				final Shell myShell=display.getActiveShell();
				display.asyncExec(new Runnable() {

					public void run() {

						new HelpBrowser().run(mapUrl,myShell);
					}

				}); 
			}
		}

		private String getLocation(String url, String meth){

			String flankMap = "";

			HttpClient client = new HttpClient();
			GetMethod method = new GetMethod(url+meth);

			// Provide custom retry handler is necessary
			method.getParams().setParameter(HttpMethodParams.RETRY_HANDLER, 
					new DefaultHttpMethodRetryHandler(3, false));

			try {
				// Execute the method.
				int statusCode = client.executeMethod(method);
				if (statusCode != HttpStatus.SC_OK) {
					displayErrorMessage(method.getStatusLine().getReasonPhrase());
				}

				// Read the response body.

				byte[] responseBody = method.getResponseBody();

				// Deal with the response.
				// Use caution: ensure correct character encoding and is not binary data
				flankMap = new String(responseBody);


			} catch (HttpException e) {
				displayErrorMessage("Fatal protocol violation: " + e.getMessage());
			} catch (IOException e) {
				displayErrorMessage("Fatal transport error: " + e.getMessage());
			} finally {
				// Release the connection.
				method.releaseConnection();
			}  

			return flankMap;
		}


		private void displayErrorMessage(String message){
			MessageBox mBox = new MessageBox(Display.getCurrent().getActiveShell(),SWT.OK);
			mBox.setText("Vista Message");
			mBox.setMessage(message + "\n Please contact your administrator\n if this is a problem");
			mBox.open();
		}
}
