/*
 * Copyright (c) 2006-2011 Massachusetts General Hospital 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the i2b2 Software License v2.1 
 * which accompanies this distribution. 
 * 
 * Contributors:
 * 	     Lori Phillips
 */

package edu.harvard.i2b2.eclipse.plugins.vista.table.views;



import java.io.IOException;
import java.io.InputStream;
import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.params.HttpMethodParams;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.ui.part.ViewPart;


/**
 * The VISTA View class provides the Vista UI View to the
 *  Eclipse framework
 * @author Lori Phillips   
 */


public class VistaView extends ViewPart {

	public static final String ID = "edu.harvard.i2b2.eclipse.plugins.vista.table.views.vistaView";
	public static final String THIS_CLASS_NAME = VistaView.class.getName();

	public static final String PREFIX = "edu.harvard.i2b2.eclipse.plugins.vista"; 
	public static final String VISTA_VIEW_CONTEXT_ID = PREFIX + ".vista_view_help_context";
	public static final String OS = System.getProperty("os.name").toLowerCase();

	private Log log = LogFactory.getLog(THIS_CLASS_NAME);
	private TableViewer viewer;


	/**
	 * The constructor
	 */
	public VistaView() {
		
	}

/*	@Override
	public void init(IViewSite site) throws PartInitException {
		super.init(site);
		// Normally we might do other stuff here.
	}
*/
	/**
	 * This is a callback that creates the view
	 * 
	 * @param parent   Composite the Vista table viewer UI is contained within
	 */
	@Override
	public void createPartControl(Composite parent) {
		Font textFont = null;
		log.info("Vista plugin version 1.0");

		if (OS.startsWith("mac"))
			textFont = new Font(parent.getDisplay(), "Monaco", 12, SWT.NORMAL);
		else
			textFont = new Font(parent.getDisplay(), "Courier New", 10, SWT.NORMAL);

		GridData gridData1 = new GridData(GridData.FILL_BOTH);
		gridData1.grabExcessVerticalSpace = true;
		gridData1.heightHint = 500;
		parent.setLayoutData(gridData1);
		
		GridLayout gridLayout = new GridLayout ();
		gridLayout.numColumns = 1;
		
		parent.setLayout(gridLayout);
		
		createViewer(parent);

	/*	Composite buttons = new Composite(parent, SWT.NONE);
		
		GridData gridData = new GridData();
		gridData.horizontalSpan =1;
		buttons.setLayoutData(gridData);
		
		GridLayout gridLayout2 = new GridLayout ();
		gridLayout2.numColumns = 2;
		buttons.setLayout(gridLayout2);


		Button getMap = new Button(buttons, SWT.PUSH);
		getMap.setText("Get Conservation Map");
		getMap.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {

				int snpStartLow = 0;
				int snpEndHigh = 0;
				String chromosome = null;
				String base = "hg18";
				
				List <SnpModel> snpsOfInterest = SnpModelProvider.getInstance().getEnabledSnps();

				// for each snp of Interest get location 

				Iterator<SnpModel> snpsIterator = snpsOfInterest.iterator();		
				while(snpsIterator.hasNext()){
					SnpModel snp = (SnpModel) snpsIterator.next();
					if(snp.getGenomicData().getChromosomeLocation().getStart() != 0){
						if ((snp.getGenomicData().getChromosomeLocation().getStart() < snpStartLow) || (snpStartLow == 0)) {
							snpStartLow = snp.getGenomicData().getChromosomeLocation().getStart();
						}
						if (snp.getGenomicData().getChromosomeLocation().getEnd() > snpEndHigh){
								snpEndHigh = snp.getGenomicData().getChromosomeLocation().getEnd();
						}
						if(chromosome == null){
							chromosome = snp.getGenomicData().getChromosomeLocation().getChromosome();
						}
					}
					else if(snp.getGenomicData().getChromosomeLocation().getStart() == 0){
						
				//		base = snp.getGenomicData().getSequenceVariantLocation().getReferenceGenomeVersion();
					
						if((snp.getGenomicData().getSequenceVariantLocation().getGeneName() != null)
								&& (snp.getGenomicData().getSequenceVariantLocation().getFlankingSeq5() != null)
								&& (snp.getGenomicData().getSequenceVariantLocation().getFlankingSeq3() != null))
						{
							// Get position
							String url = "http://hazelton.lbl.gov" ;
							String map = "/flankmap/map?"; 
							String	set =	"dataset=hg18";
							String gene = "&gene=" + snp.getGenomicData().getSequenceVariantLocation().getGeneName();
							String flank_5 ="&flank_seq_5=" + snp.getGenomicData().getSequenceVariantLocation().getFlankingSeq5();
							String flank_3 = "&flank_seq_3="  + snp.getGenomicData().getSequenceVariantLocation().getFlankingSeq3();

							String flankmap = map + set + gene + flank_5 + flank_3;

							String result = getLocation(url , flankmap);
							SAXBuilder parser = new SAXBuilder();
							java.io.StringReader xmlStringReader = new java.io.StringReader(result);

							try {
								org.jdom.Document flankMapDoc = parser.build(xmlStringReader);
								org.jdom.Element flankMapXml = flankMapDoc.getRootElement();
								Element posXml = (Element) flankMapXml.getChild("ref_position");
								
								String pos = posXml.getText().replace(",", "");
								String[] pieces = pos.split(":");	
								//	chr11:47329420-47329440";
								
								chromosome = pieces[0];
								snp.getGenomicData().getChromosomeLocation().setChromosome(pieces[0]);
								String[] endpoints = pieces[1].split("-");
								snp.getGenomicData().getChromosomeLocation().setStart(Integer.parseInt(endpoints[0]));
								snp.getGenomicData().getChromosomeLocation().setEnd(Integer.parseInt(endpoints[1]));
								
								if(snpStartLow == 0){
									snpStartLow = snp.getGenomicData().getChromosomeLocation().getStart();
								}
								else if (snp.getGenomicData().getChromosomeLocation().getStart() < snpStartLow){
									snpStartLow = snp.getGenomicData().getChromosomeLocation().getStart();
								}

								if (snp.getGenomicData().getChromosomeLocation().getEnd() > snpEndHigh){
									snpEndHigh = snp.getGenomicData().getChromosomeLocation().getEnd();
								}
							}	
							catch(Exception e1) {
								//
							}
						}
					}
				}
				
				// Now get map for entire region and display in browser.
				String url = "http://pipeline.lbl.gov";
				String method =	"/servlet/gpvista2/?output=plot&regular=0&guidelines=0&left_labels=1&contigs=0&width=800&height=40&run=80&order=2,3,11,23,362,696&pos=";
		
				if((chromosome != null) && (snpStartLow != 0) & (snpEndHigh != 0)){
					method = method + chromosome + ":" + (snpStartLow-100) + "-" + (snpEndHigh+100) + "&base=" + base ; 
			//		log.debug(method);

					final String mapUrl = url + method;
					final Button myButton = (Button) e.widget;
					Display display=myButton.getDisplay();
					final Shell myShell=myButton.getShell();
					display.asyncExec(new Runnable() {

						public void run() {

							new HelpBrowser().run(mapUrl,myShell);
						}

					}); 
				}
			}
			
		});	
		
		
		Button getPolyPhen = new Button(buttons, SWT.PUSH);
		getPolyPhen.setText("Get PolyPhen Scores");
		*/
	}	
	
	private void createViewer(Composite parent) {
		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.FULL_SELECTION);
		createColumns(viewer);
		viewer.setContentProvider(new SnpContentProvider());
		viewer.setLabelProvider(new SnpLabelProvider(viewer, Display.getCurrent()));
	    Transfer[] types = new Transfer[] { TextTransfer.getInstance() };
	    viewer.addDropSupport(DND.DROP_COPY | DND.DROP_MOVE | DND.DROP_DEFAULT, types, new SnpDropListener(viewer));
	}
	
	private void createColumns(TableViewer viewer) {

//		String[] titles = {"Select", "SNP Name", "Gene Name", "Chromosome", "Conservation Map", "PolyPhen Score", "Variant type" };
		String[] titles = {"Select", "SNP Name", "Gene Name", "Chromosome", "Conservation Map", "5' Flanking Sequence", "3' Flanking Sequence" };

		int[] bounds = { 50, 100, 100, 100, 100, 100, 100 };

		for (int i = 0; i < titles.length; i++) {
			TableViewerColumn column = new TableViewerColumn(viewer, SWT.NONE);
			column.getColumn().setText(titles[i]);
			column.getColumn().setWidth(bounds[i]);
			column.getColumn().setResizable(true);
			column.getColumn().setMoveable(true);
		
			if (i==0)
				column.setEditingSupport(new VistaCellEditingSupport(viewer, i));
			if(i == 4){
				column.setEditingSupport(new VistaCellEditingSupport(viewer, i));
			}
		}
		Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		
		GridData gridData1 = new GridData(GridData.FILL_BOTH);
		gridData1.grabExcessVerticalSpace = true;
		gridData1.heightHint = 400;
		table.setLayoutData(gridData1);
		
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}
	
	
	private String getLocation(String url, String meth){

		String flankMap = "";

		HttpClient client = new HttpClient();
		GetMethod method = new GetMethod(url+meth);

		// Provide custom retry handler is necessary
		method.getParams().setParameter(HttpMethodParams.RETRY_HANDLER, 
				new DefaultHttpMethodRetryHandler(3, false));

		try {
			// Execute the method.
			int statusCode = client.executeMethod(method);
			if (statusCode != HttpStatus.SC_OK) {
				displayErrorMessage(method.getStatusLine().getReasonPhrase());
			}

			// Read the response body.
			
			byte[] responseBody = method.getResponseBody();

			// Deal with the response.
			// Use caution: ensure correct character encoding and is not binary data
			flankMap = new String(responseBody);
	

		} catch (HttpException e) {
			displayErrorMessage("Fatal protocol violation: " + e.getMessage());
		} catch (IOException e) {
			displayErrorMessage("Fatal transport error: " + e.getMessage());
		} finally {
			// Release the connection.
			method.releaseConnection();
		}  

		return flankMap;
	}


	private Image getImage(String url, String meth){

		Image image = null;

		HttpClient client = new HttpClient();
		GetMethod method = new GetMethod(url+meth);

		// Provide custom retry handler is necessary
		method.getParams().setParameter(HttpMethodParams.RETRY_HANDLER, 
				new DefaultHttpMethodRetryHandler(3, false));

		try {
			// Execute the method.
			int statusCode = client.executeMethod(method);
			if (statusCode != HttpStatus.SC_OK) {
			//	System.err.println("Method failed: " + method.getStatusLine() );		
				displayErrorMessage(method.getStatusLine().getReasonPhrase());
				
			}

			// Read the response body.
			
			InputStream in = method.getResponseBodyAsStream();
			image = new Image(Display.getCurrent(), in);
		

		} catch (HttpException e) {
			displayErrorMessage("Fatal protocol violation: " + e.getMessage());
		} catch (IOException e) {
			displayErrorMessage("Fatal transport error: " + e.getMessage());
		} finally {
			// Release the connection.
			method.releaseConnection();
		}  

		return image;
	}
	
	
/*	private void addHelpButtonToToolBar() {
		final IWorkbenchHelpSystem helpSystem = PlatformUI.getWorkbench().getHelpSystem();
		Action helpAction = new Action() {
			@Override
			public void run(){
				helpSystem.displayHelpResource("/edu.harvard.i2b2.eclipse.plugins.vista/html/toc.html");
			}
		};	
		helpAction.setImageDescriptor(ImageDescriptor.createFromFile(VistaView.class, "/icons/help.png"));

		getViewSite().getActionBars().getToolBarManager().add(helpAction);
	}
*/
	
	
	
	private void displayErrorMessage(String message){
		  MessageBox mBox = new MessageBox(Display.getCurrent().getActiveShell(),SWT.OK);
		  mBox.setText("Vista Message");
		  mBox.setMessage(message + "\n Please contact your administrator\n if this is a problem");
		  mBox.open();
	}

	public TableViewer getViewer() {
		return viewer;
	}

	
}
